import fs from 'node:fs';
import { EngineMap } from './engine-map.js';
import { Markdown } from './engines/markdown.js';
import { Handlebars } from './engines/handlebars.js';
import { EJS } from './engines/ejs.js';
import { Pug } from './engines/pug.js';
import { Nunjucks } from './engines/nunjucks.js';
import { Liquid } from './engines/liquid.js';
export class Ecto {
    __mapping = new EngineMap();
    __engines = new Array();
    __defaultEngine = 'ejs';
    // Engines
    __ejs;
    __markdown;
    __pug;
    __nunjucks;
    __handlebars;
    __liquid;
    constructor(options) {
        const engineOptions = { ...options };
        delete engineOptions.defaultEngine;
        // Engines
        this.__ejs = new EJS(engineOptions);
        this.__markdown = new Markdown(engineOptions);
        this.__pug = new Pug(engineOptions);
        this.__nunjucks = new Nunjucks(engineOptions);
        this.__handlebars = new Handlebars(engineOptions);
        this.__liquid = new Liquid(engineOptions);
        // Register engines
        this.__engines.push(this.__ejs, this.__markdown, this.__pug, this.__nunjucks, this.__handlebars, this.__liquid);
        // Register mappings
        this.registerEngineMappings();
        // Set the options
        if (options !== undefined && this.isValidEngine(options.defaultEngine)) {
            this.__defaultEngine = options.defaultEngine;
        }
    }
    get defaultEngine() {
        return this.__defaultEngine;
    }
    set defaultEngine(value) {
        value = value.toLowerCase().trim();
        if (this.isValidEngine(value)) {
            this.__defaultEngine = value;
        }
    }
    get mappings() {
        return this.__mapping;
    }
    // Engines
    get ejs() {
        return this.__ejs;
    }
    get markdown() {
        return this.__markdown;
    }
    get pug() {
        return this.__pug;
    }
    get nunjucks() {
        return this.__nunjucks;
    }
    get handlebars() {
        return this.__handlebars;
    }
    get liquid() {
        return this.__liquid;
    }
    // String Render
    // eslint-disable-next-line max-params
    async render(source, data, engineName, rootTemplatePath, filePathOutput) {
        let result = '';
        let renderEngineName = this.__defaultEngine;
        // Set the render engine
        if (this.isValidEngine(engineName) && engineName !== undefined) {
            renderEngineName = engineName;
        }
        // Get the render engine
        const renderEngine = this.getRenderEngine(renderEngineName);
        // Set the root template path
        renderEngine.rootTemplatePath = rootTemplatePath;
        // Get the output
        result = await renderEngine.render(source, data);
        // Write out the file
        await this.writeFile(filePathOutput, result);
        return result;
    }
    // Render Sync
    // eslint-disable-next-line max-params
    renderSync(source, data, engineName, rootTemplatePath, filePathOutput) {
        let result = '';
        let renderEngineName = this.__defaultEngine;
        // Set the render engine
        if (this.isValidEngine(engineName) && engineName !== undefined) {
            renderEngineName = engineName;
        }
        // Get the render engine
        const renderEngine = this.getRenderEngine(renderEngineName);
        // Set the root template path
        renderEngine.rootTemplatePath = rootTemplatePath;
        // Get the output
        result = renderEngine.renderSync(source, data);
        // Write out the file
        this.writeFileSync(filePathOutput, result);
        return result;
    }
    // Render from File
    // eslint-disable-next-line max-params
    async renderFromFile(filePath, data, rootTemplatePath, filePathOutput, engineName) {
        let result = '';
        // Select which engine
        engineName ||= this.getEngineByFilePath(filePath);
        // Get the source
        const source = await fs.promises.readFile(filePath, 'utf8');
        result = await this.render(source, data, engineName, rootTemplatePath, filePathOutput);
        return result;
    }
    // Render from File Sync
    // eslint-disable-next-line max-params
    renderFromFileSync(filePath, data, rootTemplatePath, filePathOutput, engineName) {
        let result = '';
        // Select which engine
        engineName ||= this.getEngineByFilePath(filePath);
        // Get the source
        const source = fs.readFileSync(filePath, 'utf8');
        result = this.renderSync(source, data, engineName, rootTemplatePath, filePathOutput);
        return result;
    }
    async ensureFilePath(path) {
        const pathList = path.split('/');
        pathList.pop();
        const directory = pathList.join('/');
        if (!fs.existsSync(directory)) {
            fs.mkdirSync(directory, { recursive: true });
        }
    }
    ensureFilePathSync(path) {
        const pathList = path.split('/');
        pathList.pop();
        const directory = pathList.join('/');
        if (!fs.existsSync(directory)) {
            fs.mkdirSync(directory, { recursive: true });
        }
    }
    getEngineByFilePath(filePath) {
        let result = this.__defaultEngine;
        if (filePath !== undefined) {
            const extension = filePath.lastIndexOf('.') >= 0 ? filePath.slice(filePath.lastIndexOf('.') + 1) : '';
            const engExtension = this.__mapping.getName(extension);
            if (engExtension !== undefined) {
                result = engExtension;
            }
        }
        return result;
    }
    async findTemplateWithoutExtension(path, templateName) {
        let result = '';
        const files = await fs.promises.readdir(path);
        for (const file of files) {
            if (file.startsWith(templateName + '.')) {
                result = path + '/' + file;
                break;
            }
        }
        return result;
    }
    findTemplateWithoutExtensionSync(path, templateName) {
        let result = '';
        const files = fs.readdirSync(path);
        for (const file of files) {
            if (file.startsWith(templateName + '.')) {
                result = path + '/' + file;
                break;
            }
        }
        return result;
    }
    // Engines
    isValidEngine(engineName) {
        let result = false;
        if (engineName !== undefined && this.__mapping.get(engineName) !== undefined) {
            result = true;
        }
        return result;
    }
    registerEngineMappings() {
        for (const eng of this.__engines) {
            for (const name of eng.names) {
                this.__mapping.set(name, eng.getExtensions());
            }
        }
    }
    getRenderEngine(engineName) {
        let result = this.__ejs; // Setting default
        const cleanEngineName = engineName.trim().toLowerCase();
        switch (cleanEngineName) {
            case 'markdown': {
                result = this.__markdown;
                break;
            }
            case 'pug': {
                result = this.__pug;
                break;
            }
            case 'nunjucks': {
                result = this.__nunjucks;
                break;
            }
            case 'mustache': {
                result = this.__handlebars;
                break;
            }
            case 'handlebars': {
                result = this.__handlebars;
                break;
            }
            case 'liquid': {
                result = this.__liquid;
                break;
            }
            default: {
                result = this.__ejs;
                break;
            }
        }
        return result;
    }
    async writeFile(filePath, source) {
        if (filePath && source) {
            await this.ensureFilePath(filePath);
            await fs.promises.writeFile(filePath, source);
        }
    }
    writeFileSync(filePath, source) {
        if (filePath && source) {
            this.ensureFilePathSync(filePath);
            fs.writeFileSync(filePath, source);
        }
    }
}
//# sourceMappingURL=ecto.js.map